package cn.com.infosec.math.ec;

import java.math.BigInteger;
import java.util.Random;

public abstract class ECFieldElement implements ECConstants {
   public abstract BigInteger toBigInteger();

   public abstract String getFieldName();

   public abstract int getFieldSize();

   public abstract ECFieldElement add(ECFieldElement var1);

   public abstract ECFieldElement subtract(ECFieldElement var1);

   public abstract ECFieldElement multiply(ECFieldElement var1);

   public abstract ECFieldElement divide(ECFieldElement var1);

   public abstract ECFieldElement negate();

   public abstract ECFieldElement square();

   public abstract ECFieldElement invert();

   public abstract ECFieldElement sqrt();

   public String toString() {
      return this.toBigInteger().toString(2);
   }

   public static class F2m extends ECFieldElement {
      public static final int GNB = 1;
      public static final int TPB = 2;
      public static final int PPB = 3;
      private int representation;
      private int m;
      private int k1;
      private int k2;
      private int k3;
      private IntArray x;
      private int t;

      public F2m(int m, int k1, int k2, int k3, BigInteger x) {
         int var6;
         this.t = var6 = m + 31 >> 5;
         this.x = new IntArray(x, var6);
         if (k2 == 0 && k3 == 0) {
            this.representation = 2;
         } else {
            if (k2 >= k3) {
               throw new IllegalArgumentException("k2 must be smaller than k3");
            }

            if (k2 <= 0) {
               throw new IllegalArgumentException("k2 must be larger than 0");
            }

            this.representation = 3;
         }

         if (x.signum() >= 0) {
            this.m = m;
            this.k1 = k1;
            this.k2 = k2;
            this.k3 = k3;
         } else {
            throw new IllegalArgumentException("x value cannot be negative");
         }
      }

      public F2m(int m, int k, BigInteger x) {
         this(m, k, 0, 0, (BigInteger)x);
      }

      private F2m(int m, int k1, int k2, int k3, IntArray x) {
         this.t = m + 31 >> 5;
         this.x = x;
         this.m = m;
         this.k1 = k1;
         this.k2 = k2;
         this.k3 = k3;
         if (k2 == 0 && k3 == 0) {
            this.representation = 2;
         } else {
            this.representation = 3;
         }

      }

      public static void checkFieldElements(ECFieldElement a, ECFieldElement b) {
         if (a instanceof F2m && b instanceof F2m) {
            F2m a1 = (F2m)a;
            F2m b1 = (F2m)b;
            if (a1.m == b1.m && a1.k1 == b1.k1 && a1.k2 == b1.k2 && a1.k3 == b1.k3) {
               if (a1.representation != b1.representation) {
                  throw new IllegalArgumentException("One of the field elements are not elements has incorrect representation");
               }
            } else {
               throw new IllegalArgumentException("Field elements are not elements of the same field F2m");
            }
         } else {
            throw new IllegalArgumentException("Field elements are not both instances of ECFieldElement.F2m");
         }
      }

      public BigInteger toBigInteger() {
         return this.x.toBigInteger();
      }

      public String getFieldName() {
         return "F2m";
      }

      public int getFieldSize() {
         return this.m;
      }

      public ECFieldElement add(ECFieldElement b) {
         IntArray var2;
         (var2 = (IntArray)this.x.clone()).addShifted(((F2m)b).x, 0);
         F2m var10002 = this;
         F2m var10003 = this;
         F2m var10004 = this;
         int var5 = this.m;
         int b1 = var10004.k1;
         int var3 = var10003.k2;
         int var4 = var10002.k3;
         return new F2m(var5, b1, var3, var4, var2);
      }

      public ECFieldElement subtract(ECFieldElement b) {
         return this.add(b);
      }

      public ECFieldElement multiply(ECFieldElement b) {
         F2m b1 = (F2m)b;
         IntArray b2 = b1.x;
         IntArray var10000 = b2 = this.x.multiply(b2, this.m);
         int[] var2;
         int[] var10002 = var2 = new int[3];
         var2[0] = this.k1;
         var2[1] = this.k2;
         var10002[2] = this.k3;
         var10000.reduce(this.m, var10002);
         F2m var9 = this;
         F2m var10003 = this;
         F2m var10004 = this;
         int var5 = this.m;
         int var8 = var10004.k1;
         int var3 = var10003.k2;
         int var4 = var9.k3;
         return new F2m(var5, var8, var3, var4, b2);
      }

      public ECFieldElement divide(ECFieldElement b) {
         return this.multiply(b.invert());
      }

      public ECFieldElement negate() {
         return this;
      }

      public ECFieldElement square() {
         IntArray var1;
         IntArray var10000 = var1 = this.x.square(this.m);
         int[] var2;
         int[] var10002 = var2 = new int[3];
         var2[0] = this.k1;
         var2[1] = this.k2;
         var10002[2] = this.k3;
         var10000.reduce(this.m, var10002);
         F2m var7 = this;
         F2m var10003 = this;
         F2m var10004 = this;
         int var5 = this.m;
         int var6 = var10004.k1;
         int var3 = var10003.k2;
         int var4 = var7.k3;
         return new F2m(var5, var6, var3, var4, var1);
      }

      public ECFieldElement invert() {
         IntArray var1 = (IntArray)this.x.clone();
         IntArray var2;
         IntArray var10001 = var2 = new IntArray;
         var2.<init>(this.t);
         var2.setBit(this.m);
         var2.setBit(0);
         var10001.setBit(this.k1);
         if (this.representation == 3) {
            var2.setBit(this.k2);
            var2.setBit(this.k3);
         }

         IntArray var3;
         IntArray var10000 = var3 = new IntArray;
         var10000.<init>(this.t);
         var10000.setBit(0);
         IntArray var4;
         var4 = new IntArray.<init>(this.t);

         while(!var1.isZero()) {
            int var5;
            if ((var5 = var1.bitLength() - var2.bitLength()) < 0) {
               var5 = -var5;
               var10000 = var1;
               IntArray var10002 = var3;
               var3 = var4;
               var4 = var10002;
               var1 = var2;
               var2 = var10000;
            }

            int var10004 = var5;
            var5 >>= 5;
            int var6;
            var1.addShifted(var2.shiftLeft(var6 = var10004 & 31), var5);
            var3.addShifted(var4.shiftLeft(var6), var5);
         }

         F2m var11 = this;
         F2m var10003 = this;
         F2m var12 = this;
         int var7 = this.m;
         int var8 = var12.k1;
         int var9 = var10003.k2;
         int var10 = var11.k3;
         return new F2m(var7, var8, var9, var10, var4);
      }

      public ECFieldElement sqrt() {
         throw new RuntimeException("Not implemented");
      }

      public int getRepresentation() {
         return this.representation;
      }

      public int getM() {
         return this.m;
      }

      public int getK1() {
         return this.k1;
      }

      public int getK2() {
         return this.k2;
      }

      public int getK3() {
         return this.k3;
      }

      public boolean equals(Object anObject) {
         if (anObject == this) {
            return true;
         } else if (!(anObject instanceof F2m)) {
            return false;
         } else {
            F2m anObject1 = (F2m)anObject;
            return this.m == anObject1.m && this.k1 == anObject1.k1 && this.k2 == anObject1.k2 && this.k3 == anObject1.k3 && this.representation == anObject1.representation && this.x.equals(anObject1.x);
         }
      }

      public int hashCode() {
         return this.x.hashCode() ^ this.m ^ this.k1 ^ this.k2 ^ this.k3;
      }
   }

   public static class Fp extends ECFieldElement {
      BigInteger x;
      BigInteger q;

      public Fp(BigInteger q, BigInteger x) {
         this.x = x;
         if (x.compareTo(q) < 0) {
            this.q = q;
         } else {
            throw new IllegalArgumentException("x value too large in field element");
         }
      }

      private static BigInteger[] lucasSequence(BigInteger p, BigInteger P, BigInteger Q, BigInteger k) {
         int var10000 = k.bitLength();
         int var4 = k.getLowestSetBit();
         BigInteger var5 = ECConstants.ONE;
         BigInteger var6 = ECConstants.TWO;
         int var7 = var10000 - 1;
         BigInteger var10 = P;
         BigInteger var8 = var6;
         BigInteger var12 = var5;
         var6 = var5;

         for(int var9 = var7; var9 >= var4 + 1; --var9) {
            var5 = var5.multiply(var6).mod(p);
            if (k.testBit(var9)) {
               var6 = var5.multiply(Q).mod(p);
               var12 = var12.multiply(var10).mod(p);
               var8 = var10.multiply(var8).subtract(P.multiply(var5)).mod(p);
               var10 = var10.multiply(var10).subtract(var6.shiftLeft(1)).mod(p);
            } else {
               var12 = var12.multiply(var8).subtract(var5).mod(p);
               var10 = var10.multiply(var8).subtract(P.multiply(var5)).mod(p);
               var8 = var8.multiply(var8).subtract(var5.shiftLeft(1)).mod(p);
               var6 = var5;
            }
         }

         BigInteger var13 = k = var5.multiply(var6).mod(p);
         Q = k.multiply(Q).mod(p);
         var5 = var12.multiply(var8).subtract(k).mod(p);
         P = var10.multiply(var8).subtract(P.multiply(k)).mod(p);
         Q = var13.multiply(Q).mod(p);

         for(int k1 = 1; k1 <= var4; ++k1) {
            var5 = var5.multiply(P).mod(p);
            P = P.multiply(P).subtract(Q.shiftLeft(1)).mod(p);
            Q = Q.multiply(Q).mod(p);
         }

         return new BigInteger[]{var5, P};
      }

      public BigInteger toBigInteger() {
         return this.x;
      }

      public String getFieldName() {
         return "Fp";
      }

      public int getFieldSize() {
         return this.q.bitLength();
      }

      public BigInteger getQ() {
         return this.q;
      }

      public ECFieldElement add(ECFieldElement b) {
         return new Fp(this.q, this.x.add(b.toBigInteger()).mod(this.q));
      }

      public ECFieldElement subtract(ECFieldElement b) {
         return new Fp(this.q, this.x.subtract(b.toBigInteger()).mod(this.q));
      }

      public ECFieldElement multiply(ECFieldElement b) {
         return new Fp(this.q, this.x.multiply(b.toBigInteger()).mod(this.q));
      }

      public ECFieldElement divide(ECFieldElement b) {
         return new Fp(this.q, this.x.multiply(b.toBigInteger().modInverse(this.q)).mod(this.q));
      }

      public ECFieldElement negate() {
         return new Fp(this.q, this.x.negate().mod(this.q));
      }

      public ECFieldElement square() {
         BigInteger var1 = this.q;
         BigInteger var10003 = this.x;
         return new Fp(var1, var10003.multiply(var10003).mod(this.q));
      }

      public ECFieldElement invert() {
         BigInteger var10002 = this.q;
         return new Fp(var10002, this.x.modInverse(var10002));
      }

      public ECFieldElement sqrt() {
         if (!this.q.testBit(0)) {
            throw new RuntimeException("not done yet");
         } else {
            BigInteger var2;
            if (this.q.testBit(1)) {
               Fp var9;
               Fp var11 = var9 = new Fp;
               BigInteger var10002 = var2 = this.q;
               BigInteger var10003 = this.x;
               var2 = var2.shiftRight(2).add(ECConstants.ONE);
               var11.<init>(var10002, var10003.modPow(var2, this.q));
               if (!var11.square().equals(this)) {
                  var9 = null;
               }

               return var9;
            } else {
               BigInteger var1;
               BigInteger var3 = (var2 = this.q.subtract(var1 = ECConstants.ONE)).shiftRight(1);
               if (!this.x.modPow(var3, this.q).equals(var1)) {
                  return null;
               } else {
                  var1 = var2.shiftRight(2).shiftLeft(1).add(var1);
                  BigInteger var4;
                  BigInteger var5 = (var4 = this.x).shiftLeft(2).mod(this.q);
                  Random var6;
                  var6 = new Random.<init>();

                  while(true) {
                     BigInteger var7;
                     BigInteger var10000;
                     do {
                        var10000 = var7 = new BigInteger;
                        var7.<init>(this.q.bitLength(), var6);
                     } while(var10000.compareTo(this.q) >= 0);

                     if (var7.multiply(var7).subtract(var5).modPow(var3, this.q).equals(var2)) {
                        BigInteger[] var10 = lucasSequence(this.q, var7, var4, var1);
                        var7 = var10[0];
                        BigInteger var8;
                        var10000 = var8 = var10[1];
                        if (var10000.multiply(var10000).mod(this.q).equals(var5)) {
                           if (var8.testBit(0)) {
                              var8 = var8.add(this.q);
                           }

                           var1 = var8.shiftRight(1);
                           return new Fp(this.q, var1);
                        }

                        if (!var7.equals(ECConstants.ONE) && !var7.equals(var2)) {
                           return null;
                        }
                     }
                  }
               }
            }
         }
      }

      public boolean equals(Object other) {
         if (other == this) {
            return true;
         } else if (!(other instanceof Fp)) {
            return false;
         } else {
            Fp other1 = (Fp)other;
            return this.q.equals(other1.q) && this.x.equals(other1.x);
         }
      }

      public int hashCode() {
         return this.q.hashCode() ^ this.x.hashCode();
      }
   }
}
